/*
 * logping v1.0
 *
 * Log icmp packets
 *
 * October, 1997
 * Neil Schemenauer <nascheme@acs.ucalgary.ca>
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define MAXPACKET 4096

void print_packet(char *buf, int size, struct sockaddr *from);

int main()
{
	struct protoent *proto;	/* protocol entry */
	int sock;		/* raw icmp socket */
	struct sockaddr from;   /* address of sender */
	int from_len;		/* length of from address */
	char buf[MAXPACKET];	/* buffer to store packet data */
	int n;			/* number of bytes read from socket */

	/* 
	 * setup raw socket
	 */
	if ( (proto = getprotobyname("icmp")) == NULL) {
		fprintf(stderr, "Error: unknown protocol icmp\n");
		exit(-1);
	}
	if ( (sock = socket(AF_INET, SOCK_RAW, proto->p_proto)) < 0) {
		fprintf(stderr, "Error: can't create raw socket (root priviledge needed)\n");
		exit(-1);
	}

	/*
	 * loop forever reading packets
         */
	for ( ; ; ) {
		if ( (n = recvfrom(sock, buf, sizeof(buf), 0, &from, &from_len)) < 0 ) {
			fprintf(stderr, "Error: recvfrom error %d\n", errno);
			exit(-1);
		}
		print_packet(buf, n, &from);
	}

	return 0;
}


void print_packet(char *buf, int size, struct sockaddr *from)
{
	struct ip *ip;
	struct icmphdr *icmphdr;
	struct sockaddr_in *from_in;
	
	ip = (struct ip*)buf;
	from_in = (struct sockaddr_in *)from;
	icmphdr = (struct icmphdr*)(buf+(ip->ip_hl << 2));

	/*
	printf("packet size %d\n", size);
	printf("ip header size %d\n", ip->ip_hl << 2);
	printf("remote address %s\n", inet_ntoa(from_in->sin_addr));
	printf("remote port %hd\n", ntohs(from_in->sin_port));

	printf("icmp type %hd\n", icmphdr->type);
	printf("icmp code %hd\n", icmphdr->code);
	printf("icmp sequence %d\n", icmphdr->un.echo.sequence);
	*/

	/* print out ping requests */
	if (icmphdr->type == ICMP_ECHO) {
		printf("ICMP_ECHO (ping request) from %s\n", inet_ntoa(from_in->sin_addr));
	}
}
